#include <formatio.h>
#include <ansi_c.h>
#include <visa.h>
#include <cvirte.h>     
#include <userint.h>
#include "rsafq.h"
#include "RSAFQ_Trigger_Example.h"


	/* Values for control Active Output */
#define RSAFQ_OUTPUT_TYPE_BALANCED		0
#define RSAFQ_OUTPUT_TYPE_UNBALANCED	1

	/* Values for control Active Output */
#define RSAFQ_ACTIVE_OUTPUT_ANALOG1		1
#define RSAFQ_ACTIVE_OUTPUT_ANALOG2		2

#define RSAFQ_SEGMENT_MODE_CONT         0

#define RSAFQ_SEGMENT_SOURCE_AUTO       2
#define RSAFQ_SEGMENT_SOURCE_MAN        1
#define RSAFQ_SEGMENT_SOURCE_BUS		0

/* Error check macro for driver functions */

#define CHECKERR( fCal ) \
    { \
        error = (fCal);\
        rsafq_error_message (io, error, error_message);\
        if (error < VI_SUCCESS) {\
            sprintf (error_buffer, "Primary Error: 0x%08X, %s\n", error, error_message);\
            MessagePopup ("Error", error_buffer); \
            rsafq_error_query (io, &error, error_message);\
            sprintf (error_buffer, "Secondary Error: 0x%08X, %s\n", error, error_message);\
            SetCtrlAttribute (panelHandle, PANEL_APPLY, ATTR_DIMMED, 0);\
            SetCtrlAttribute (panelHandle, PANEL_Exit, ATTR_DIMMED, 0);\
            SetWaitCursor (0); \
            MessagePopup ("Error", error_buffer); \
            rsafq_close(io); \
            io = NULL;\
            return -1; \
        } \
    }
    
/* Globals */
static int panelHandle;

ViStatus  error;
ViSession io = NULL;
ViChar    error_message[256];
ViChar    error_buffer[256];

/* Local Functions */
int initialize (void);

int main (int argc, char *argv[])
{
    if (InitCVIRTE (0, argv, 0) == 0)
        return -1;  /* out of memory */
    if ((panelHandle = LoadPanel (0, "RSAFQ_Trigger_Example.uir", PANEL)) < 0)
        return -1;

	DisplayPanel (panelHandle);
    RunUserInterface ();
    DiscardPanel (panelHandle);
   
    return 0;
}

/*===========================================================================*/
/* Function: Apply Settings                                                  */
/* Purpose:  This is a callback function of Apply button. The values are     */
/*           send to the instrument                                          */
/*===========================================================================*/
int CVICALLBACK apllySettings (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    ViInt32	    outState,
				trigMode,
				/*bRetrigger,*/
				trigSource,
				segmMode,
				segmSource;
    ViStatus    rsafq_status = VI_SUCCESS;
    ViChar      wave_path[1024];
    ViInt32     file_size;
	ViInt32 	arbState, 
				/*actOutput, */
				outputType, 
				activeOutput,
				ampMode,
				bBiasEnabled;
	ViReal64	amplitude,
				iOffset,
				qOffset,
				bias;

	ViChar      buffer[1024];
	ViInt32     error_code;	
	
	
    switch (event)
    {
        case EVENT_COMMIT:
		{
            GetCtrlVal (panelHandle, PANEL_PATH, wave_path); 
            if (GetFileInfo (wave_path, &file_size)==0)
            {
            	MessagePopup ("ERROR!", "File doesn't exist");
                return 0;
            }   			

			SetCtrlAttribute (panelHandle, PANEL_APPLY, ATTR_DIMMED, 1);
			SetCtrlAttribute (panelHandle, PANEL_Exit, ATTR_DIMMED, 1);
			SetWaitCursor (1);
			
			ProcessDrawEvents ();
		
			if (io == NULL)
	            if (initialize() < VI_SUCCESS)
	            {
	          	   	SetWaitCursor (0);
	          	   	break;
	            } 

			/* 
			 *	check time out by checking the length of the file
			 *  read first - set it back later
			 */
			viSetAttribute (io, VI_ATTR_TMO_VALUE, 200000);	

			GetCtrlVal(panelHandle, PANEL_OUTPUT_STATE, &outState);
			GetCtrlVal(panelHandle, PANEL_ARB_STATE, &arbState);
			//GetCtrlVal(panelHandle, PANEL_ACTIVE_OUTPUT, &actOutput);
			GetCtrlVal(panelHandle, PANEL_OUTPUT_TYPE, &outputType);
			GetCtrlVal(panelHandle, PANEL_OUTPUT_AMPLITUDE, &amplitude);
			GetCtrlVal(panelHandle, PANEL_AMPLITUDE_MODE, &ampMode);
			GetCtrlVal(panelHandle, PANEL_OUTPUT_I_OFFSET, &iOffset);
			GetCtrlVal(panelHandle, PANEL_OUTPUT_Q_OFFSET, &qOffset);
			GetCtrlVal(panelHandle, PANEL_ENABLE_BIAS, &bBiasEnabled);
			GetCtrlVal(panelHandle, PANEL_OUTPUT_BIAS, &bias);

			GetCtrlVal(panelHandle, PANEL_TRIG_MODE, &trigMode);
			//GetCtrlVal(panelHandle, PANEL_ENABLE_RETRIG, &bRetrigger);
			GetCtrlVal(panelHandle, PANEL_TRIG_SOURCE, &trigSource);			
			GetCtrlVal(panelHandle, PANEL_SEGM_MODE, &segmMode);
			GetCtrlVal(panelHandle, PANEL_SEGM_SOURCE, &segmSource);			
			
			CHECKERR (rsafq_reset (io));
			
			CHECKERR( rsafq_MMEMoryDrive (io, "D:\\"));
			CHECKERR( rsafq_MMEMoryChangeDirectory (io, ".."));
			CHECKERR( rsafq_MMEMoryMakeDirectory (io, "example"));
			
		    /*
		     * copy file to AFQ
			 * add destination file in UIR 
		     * checked with wlan.wv			
		     */
		    CHECKERR (rsafq_writeFromFile(io, wave_path, "d:\\example\\example.wv"));

			/* Configure Trigger */
			CHECKERR (rsafq_ConfigureTrigger(io, trigMode, VI_TRUE, trigSource, 0, 0, 
											 RSAFQ_INPUT_TRIGGER_SLOPE_POSITIVE));
			
			CHECKERR (rsafq_ARBWaveformSelect(io, "d:\\example\\example.wv"));   

			CHECKERR (rsafq_error_query(io, &error_code,buffer));
		
		    CHECKERR (rsafq_SetARBState(io, arbState)); 
			
			/* Configure Segment Trigger */
			CHECKERR (rsafq_ConfigureSegmentTrigger(io, segmMode, segmSource, 0, 0, 
													RSAFQ_INPUT_TRIGGER_SEGMENT_SLOPE_POSITIVE));

			/*Selecting digital output*/
			CHECKERR (rsafq_SetActiveOutput (io, RSAFQ_OUTPUT_ANALOG));
				
			/* Configure Analog Output */
			CHECKERR( rsafq_ConfigureAnalogOutput(io, outState, 1, outputType, amplitude, 
								ampMode, iOffset/1000, qOffset/1000, bBiasEnabled, bias));
			
			SetCtrlAttribute (panelHandle, PANEL_APPLY, ATTR_DIMMED, 0);
			SetCtrlAttribute (panelHandle, PANEL_Exit, ATTR_DIMMED, 0);
			
			SetWaitCursor (0);
			
		} break;
    }
    return 0;
}


/*===========================================================================*/
/* Function: Panel Help                                                      */
/* Purpose:  This is a callback function of the menu bar. It displays a help */
/*           window describing panel being used.                             */
/*===========================================================================*/
void CVICALLBACK panelHelp (int menuBar, int menuItem, void *callbackData,
        int panel)
{
    if (panel == panelHandle)
        MessagePopup ("Help", "This is a simple demo program using\n"
                              "the Rohde&Schwarz AFQ Modulation Generator\n"
                              "instrument driver provided in the package.\n");
}

/*===========================================================================*/
/* Function: Exit                                                            */
/* Purpose:  This is a callback function of Exit button. It closes the       */
/*           session and exits the example.                                  */
/*===========================================================================*/
int CVICALLBACK exitApp (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event)
    {
        case EVENT_COMMIT:
			if(io != NULL)
				rsafq_close (io);
            QuitUserInterface (0);
            break;
    }
    return 0;
}

/*===========================================================================*/
/* Function: pathButton                                                      */ 
/* Purpose:  This is a callback function of Path button.                     */
/*===========================================================================*/
int CVICALLBACK pathButton (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    long ret_val;
    char wave_path[1024];
    
    switch (event)
    {
        case EVENT_COMMIT:
		{
            ret_val = FileSelectPopup ("", "*.wv", "*.wv", "Select Waveform File",
                                       VAL_SELECT_BUTTON, 0, 0, 1, 0, (char *) wave_path);
            if (ret_val >0) SetCtrlVal (panelHandle, PANEL_PATH, wave_path); 
        } break;
    }
	
    return 0;
}


/*===========================================================================*/
/* Function: outTypeRing                                                     */
/* Purpose:  This is a callback function of Type Ring.                       */
/*===========================================================================*/
int CVICALLBACK outTypeRing (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
	ViInt32		outType,
				bBiasEnabled;
	ViBoolean	bDimmed;
	ViReal64	amplitude;

    switch (event)
    {
        case EVENT_COMMIT:
		{
			GetCtrlIndex (panelHandle, PANEL_OUTPUT_TYPE, &outType);				
			GetCtrlVal(panelHandle, PANEL_OUTPUT_AMPLITUDE, &amplitude);
			
			switch(outType)
			{
				case RSAFQ_OUTPUT_TYPE_UNBALANCED:
					bDimmed = VI_TRUE;
					if (amplitude < 0 || amplitude > 0.75) amplitude = 0.5;
					break;
				case RSAFQ_OUTPUT_TYPE_BALANCED:
					bDimmed = VI_FALSE;
					if (amplitude < 0 || amplitude > 1.5) amplitude = 1.0;
					break;
			}
						
			SetCtrlAttribute (panelHandle, PANEL_TEXTMSG, ATTR_DIMMED, bDimmed);								
			SetCtrlAttribute (panelHandle, PANEL_ENABLE_BIAS, ATTR_DIMMED, bDimmed);
			SetCtrlVal(panelHandle, PANEL_OUTPUT_AMPLITUDE, amplitude);
			
			GetCtrlVal(panelHandle, PANEL_ENABLE_BIAS, &bBiasEnabled);			
			if (bBiasEnabled)
			{				
				SetCtrlAttribute (panelHandle, PANEL_OUTPUT_BIAS, ATTR_DIMMED, bDimmed);				
			}
        } break;
    }
    return 0;
}


/*===========================================================================*/
/* Function: biasCheckBox													 */
/* Purpose:  This is a callback function of Bias CheckBox.                   */
/*===========================================================================*/
int CVICALLBACK biasCheckBox (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
	ViInt32		bBiasEnabled;

    switch (event)
        {
        case EVENT_COMMIT:
			{
			GetCtrlVal(panelHandle, PANEL_ENABLE_BIAS, &bBiasEnabled);			

			SetCtrlAttribute (panelHandle, PANEL_OUTPUT_BIAS, ATTR_DIMMED, !bBiasEnabled);				
            } break;
        }
    return 0;
}

/*===========================================================================*/ 
/* Local Functions Definitions                                               */
/*===========================================================================*/

/*===========================================================================*/
/* Function: Initialize                                                      */
/* Purpose:  This function initializes the instrument, performs ID Query     */
/*           and Reset                                                       */
/*===========================================================================*/
int initialize (void)
{
    ViChar      instr_desc[256];
    ViBoolean   id;
    ViBoolean   rst;


    SetWaitCursor(0);
    GetCtrlVal (panelHandle, PANEL_INSTR_DESC, instr_desc);
    GetCtrlVal (panelHandle, PANEL_IDQUERY, &id);
    GetCtrlVal (panelHandle, PANEL_RESET, &rst);
    SetWaitCursor(1);
    if ((error = rsafq_init (instr_desc, id, rst, &io)) < 0 )
    {
        rsafq_error_message(VI_NULL, error, error_message);
        sprintf (error_buffer, "Initialization Error:\n\n%s\n\nCheck your "
                "connections and make sure you have the right instrument descriptor.",
                 error_message);
        MessagePopup ("ERROR!", error_buffer);
    		SetCtrlAttribute (panelHandle, PANEL_APPLY, ATTR_DIMMED, 0);
		SetCtrlAttribute (panelHandle, PANEL_Exit, ATTR_DIMMED, 0);
        SetWaitCursor(0);
        return(-1);   
    }

    viSetAttribute (io, VI_ATTR_TMO_VALUE, 10000);

    SetWaitCursor(0);
    return 0;
}

/*===========================================================================*/
/* Function: segmModeRing                                                     */
/* Purpose:  This is a callback function of Type Ring.                       */
/*===========================================================================*/
int CVICALLBACK segmModeRing (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	ViInt32		segmMode, segmSrc;
	
	switch (event)
	{
		case EVENT_COMMIT:
		{
			GetCtrlIndex (panelHandle, PANEL_SEGM_MODE, &segmMode);
			GetCtrlIndex (panelHandle, PANEL_SEGM_SOURCE, &segmSrc);
			
			if ((segmMode == RSAFQ_SEGMENT_MODE_CONT) && (segmSrc == RSAFQ_SEGMENT_SOURCE_BUS))
			{
				SetCtrlVal (panelHandle, PANEL_SEGM_SOURCE, 0);
				SetCtrlAttribute (panelHandle, PANEL_SEGM_TRG_EXEC, ATTR_DIMMED, 1);
			}
			
			if ((segmMode == RSAFQ_SEGMENT_MODE_CONT) && (segmSrc == RSAFQ_SEGMENT_SOURCE_MAN))
			{
				SetCtrlVal (panelHandle, PANEL_SEGM_SOURCE, 1);
				SetCtrlAttribute (panelHandle, PANEL_SEGM_TRG_EXEC, ATTR_DIMMED, 0);
			}
			
			if ((segmMode == RSAFQ_SEGMENT_MODE_CONT) && (segmSrc == RSAFQ_SEGMENT_SOURCE_AUTO))
			{
				SetCtrlVal (panelHandle, PANEL_SEGM_SOURCE, 0);
				SetCtrlAttribute (panelHandle, PANEL_SEGM_TRG_EXEC, ATTR_DIMMED, 1);
			}
		}
		break;
	}
	return 0;
}

/*===========================================================================*/
/* Function: segmSrcRing                                                     */
/* Purpose:  This is a callback function of Type Ring.                       */
/*===========================================================================*/
int CVICALLBACK segmSrcRing (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	ViInt32		segmMode, segmSrc;
	
	switch (event)
	{
		case EVENT_COMMIT:
		{
			GetCtrlIndex (panelHandle, PANEL_SEGM_MODE, &segmMode);				
			GetCtrlIndex (panelHandle, PANEL_SEGM_SOURCE, &segmSrc);				
			
			if ((segmMode == RSAFQ_SEGMENT_MODE_CONT) && (segmSrc == RSAFQ_SEGMENT_SOURCE_AUTO))
				SetCtrlVal (panelHandle, PANEL_SEGM_SOURCE, 0);
			
			if (segmSrc == RSAFQ_SEGMENT_SOURCE_MAN)
				SetCtrlAttribute (panelHandle, PANEL_SEGM_TRG_EXEC, ATTR_DIMMED, 0);
			else
				SetCtrlAttribute (panelHandle, PANEL_SEGM_TRG_EXEC, ATTR_DIMMED, 1);
		}
		break;
	}
	return 0;
}

/*===========================================================================*/
/* Function: trigSrcRing                                                     */
/* Purpose:  This is a callback function of Type Ring.                       */
/*===========================================================================*/
int CVICALLBACK trigSrcRing (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	ViInt32		segmSrc;
	
	switch (event)
	{
		case EVENT_COMMIT:
		{
			GetCtrlIndex (panelHandle, PANEL_TRIG_SOURCE, &segmSrc);				
			
			if (segmSrc == RSAFQ_SEGMENT_SOURCE_MAN)
				SetCtrlAttribute (panelHandle, PANEL_TRG_EXEC, ATTR_DIMMED, 0);
			else
				SetCtrlAttribute (panelHandle, PANEL_TRG_EXEC, ATTR_DIMMED, 1);
		}
		break;
	}
	return 0;
}

/*===========================================================================*/
/* Function: trgExecButton                                                   */
/* Purpose:  This is a callback function of Trigger Execute Button           */
/*===========================================================================*/
int CVICALLBACK trgExecButton (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event)
	{
		case EVENT_COMMIT:
		{
			if (io == NULL)
			{
            	MessagePopup ("Warning!", "Apply the settings first.");
                return 0;
            }
			
			CHECKERR (rsafq_TriggerExecute (io));
		}
		break;
	}
	return 0;
}

/*===========================================================================*/
/* Function: segmTrgExecButton                                               */
/* Purpose:  This is a callback function of Segment Trigger Execute Button   */
/*===========================================================================*/
int CVICALLBACK segmTrgExecButton (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event)
	{
		case EVENT_COMMIT:
		{
			if (io == NULL)
			{
            	MessagePopup ("Warning!", "Apply the settings first.");
                return 0;
            }
			
			CHECKERR (rsafq_SegmentTriggerExecute (io));
		}
		break;
	}
	return 0;
}

/*===========================================================================*/
/* Function: outpAmplCheck                                                   */
/* Purpose:  This is a callback function of Output Amplitude                 */
/*===========================================================================*/
int CVICALLBACK outpAmplCheck (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	ViInt32		outType;
	ViReal64	amplitude;

	switch (event)
	{
		case EVENT_COMMIT:
		{
			GetCtrlIndex (panelHandle, PANEL_OUTPUT_TYPE, &outType);				
			GetCtrlVal(panelHandle, PANEL_OUTPUT_AMPLITUDE, &amplitude);
			
			switch(outType)
			{
				case RSAFQ_OUTPUT_TYPE_UNBALANCED:
					if (amplitude < 0) amplitude = 0.0; 
					if (amplitude > 0.75) amplitude = 0.75;
					break;
				case RSAFQ_OUTPUT_TYPE_BALANCED:
					if (amplitude < 0) amplitude = 0.0; 
					if (amplitude > 1.5) amplitude = 1.5;
					break;
			}
						
			SetCtrlVal(panelHandle, PANEL_OUTPUT_AMPLITUDE, amplitude);
		}
		break;
	}
	return 0;
}
